#! /usr/bin/env python
# -*- coding: utf-8 -*-

from matplotlib._api import kwarg_error
import numpy as np
import pandas as pd

from saqc.constants import UNFLAGGED
from saqc.core.flags import Flags
from saqc.core.frame import DictOfSeries
from saqc.core.history import History
from saqc.core.translation.basescheme import MappingScheme
from saqc.lib.tools import getUnionIndex


class FloatScheme(MappingScheme):
    """
    Acts as the default Translator, provides a changeable subset of the
    internal float flags
    """

    _MAP = {
        np.nan: np.nan,
        -np.inf: -np.inf,
        **{k: k for k in np.arange(0, 256, dtype=float)},
    }

    def __init__(self):
        super().__init__(forward=self._MAP, backward=self._MAP)


class AnnotatedFloatScheme(FloatScheme):
    def toExternal(self, flags: Flags, attrs: dict | None = None) -> DictOfSeries:

        tflags = super().toExternal(flags, attrs=attrs)

        out = DictOfSeries()
        for field in tflags.columns:
            df = pd.DataFrame(
                {
                    "flag": tflags[field],
                    "func": "",
                    "parameters": "",
                }
            )

            history = flags.history[field]

            for col in history.columns:
                valid = (history.hist[col] != UNFLAGGED) & history.hist[col].notna()
                meta = history.meta[col]
                df.loc[valid, "func"] = meta["func"]
                df.loc[valid, "parameters"] = str(meta["kwargs"])
                out[field] = df

        return out

    def toInternal(self, flags: DictOfSeries) -> Flags:
        data = {}
        for key, frame in flags.items():
            history = History(index=frame.index)
            for (flag, func, kwargs), values in frame.groupby(["flag", "func", "parameters"]):
                column = pd.Series(np.nan, index=frame.index)
                column.loc[values.index] = self(flag)
                history.append(column, meta={"func": func, "kwargs": kwargs})
            data[key] = history
        return Flags(data)
